@@ -25,12 +25,17 @@ LOG = logging.getLogger(__name__)
 
 
 class BaseCommand(api.Command):
+    READ_ONLY = False
+
     def __init__(self, api):
         self.api = api
         self.result = None
 
     def execute(self, check_error=False, log_errors=True):
         try:
+            if self.READ_ONLY:
+                self.run_idl(None)
+                return self.result
             with self.api.transaction(check_error, log_errors) as txn:
                 txn.add(self)
             return self.result
@@ -60,6 +65,10 @@ class BaseCommand(api.Command):
                       if k not in ['api', 'result']))
 
 
+class ReadOnlyCommand(BaseCommand):
+    READ_ONLY = True
+
+
 class AddCommand(BaseCommand):
     table_name = []  # unhashable, won't be looked up
 
@@ -171,7 +180,7 @@ class DbClearCommand(BaseCommand):
         setattr(record, self.column, value)
 
 
-class DbGetCommand(BaseCommand):
+class DbGetCommand(ReadOnlyCommand):
     def __init__(self, api, table, record, column):
         super(DbGetCommand, self).__init__(api)
         self.table = table
@@ -191,7 +200,7 @@ class DbGetCommand(BaseCommand):
             self.result = result
 
 
-class DbListCommand(BaseCommand):
+class DbListCommand(ReadOnlyCommand):
     def __init__(self, api, table, records, columns, if_exists, row=False):
         super(DbListCommand, self).__init__(api)
         self.table = table
@@ -254,7 +263,7 @@ class DbListCommand(BaseCommand):
                                        match=records_found[0])
 
 
-class DbFindCommand(BaseCommand):
+class DbFindCommand(ReadOnlyCommand):
     def __init__(self, api, table, *conditions, **kwargs):
         super(DbFindCommand, self).__init__(api)
         self.table = self.api._tables[table]
@@ -274,7 +283,7 @@ class DbFindCommand(BaseCommand):
         ]
 
 
-class BaseGetRowCommand(BaseCommand):
+class BaseGetRowCommand(ReadOnlyCommand):
     def __init__(self, api, record):
         super(BaseGetRowCommand, self).__init__(api)
         self.record = record
