@@ -86,9 +86,16 @@ class API(api.API):
         """
 
     @abc.abstractmethod
-    def ls_remove_dns_record(self, switch_uuid, dns_uuid):
+    def ls_remove_dns_record(self, switch_uuid, dns_uuid, if_exists=False):
         """Remove the 'dns_record' from the switch's 'dns_records' list
 
+        :param switch_uuid: The uuid of the switch
+        :type switch_uuid:  string or uuid.UUID
+        :param dns_uuid:    The uuid of the DNS record
+        :type dns_uuid:     string or uuid.UUID
+        :param if_exists:   If True, don't fail if the DNS record
+                            doesn't exist
+        :type if_exists:    boolean
         :returns: :class:`Command` with RowView result
         """
 
@@ -140,6 +147,111 @@ class API(api.API):
         :returns:      :class:`Command` with RowView list result
         """
 
+    @abc.abstractmethod
+    def pg_acl_add(self, port_group, direction, priority, match, action,
+                   log=False):
+        """Add an ACL to 'port_group'
+
+        :param port_group: The name or uuid of the port group
+        :type port_group:  string or uuid.UUID
+        :param direction:  The traffic direction to match
+        :type direction:   'from-lport' or 'to-lport'
+        :param priority:   The priority field of the ACL
+        :type priority:    int
+        :param match:      The match rule
+        :type match:       string
+        :param action:     The action to take upon match
+        :type action:      'allow', 'allow-related', 'drop', or 'reject'
+        :param log:        If True, enable packet logging for the ACL
+        :type log:         boolean
+        :returns:          :class:`Command` with RowView result
+        """
+
+    @abc.abstractmethod
+    def pg_acl_del(self, port_group, direction=None, priority=None,
+                   match=None):
+        """Remove ACLs from 'port_group'
+
+        If only port_group is supplied, all the ACLs from the logical switch
+        are deleted. If direction is also specified, then all the flows in
+        that direction will be deleted from the Port Group. If all the fields
+        are given, then only flows that match all fields will be deleted.
+
+        :param port_group: The name or uuid of the port group
+        :type port_group:  string or uuid.UUID
+        :param direction:  The traffic direction to match
+        :type direction:   'from-lport' or 'to-lport'
+        :param priority:   The priority field of the ACL
+        :type priority:    int
+        :param match:      The match rule
+        :type match:       string
+        :returns:          :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def pg_acl_list(self, port_group):
+        """Get the ACLs for 'port group'
+
+        :param port_group: The name or uuid of the switch
+        :type port_group:  string or uuid.UUID
+        :returns:          :class:`Command` with RowView list result
+        """
+
+    @abc.abstractmethod
+    def qos_add(self, switch, direction, priority, match, rate=None,
+                burst=None, dscp=None, may_exist=False, **columns):
+        """Add an Qos rules to 'switch'
+
+        :param switch:    The name or uuid of the switch
+        :type switch:     string or uuid.UUID
+        :param direction: The traffic direction to match
+        :type direction:  'from-lport' or 'to-lport'
+        :param priority:  The priority field of the QoS
+        :type priority:   int
+        :param match:     The match rule
+        :type match:      string
+        :param dscp:      The dscp mark to take upon match
+        :type dscp:       int
+        :param rate:      The rate limit to take upon match
+        :type rate:       int
+        :param burst:     The burst rate limit to take upon match
+        :type burst:      int
+        :param may_exist: If True, don't fail if the QoS rule already exists
+        :type may_exist:  boolean
+        :param columns:   Additional columns to directly set on the switch
+        :returns:         :class:`Command` with RowView result
+        """
+
+    @abc.abstractmethod
+    def qos_del(self, switch, direction=None, priority=None, match=None):
+        """Remove Qos rules from 'switch'
+
+        If only switch is supplied, all the QoS rules from the logical switch
+        are deleted. If direction is also specified, then all the flows in
+        that direction will be deleted from the logical switch. If all the
+        fields are given, then only flows that match all fields will be
+        deleted.
+
+        :param switch:    The name or uuid of the switch
+        :type switch:     string or uuid.UUID
+        :param direction: The traffic direction to match
+        :type direction:  'from-lport' or 'to-lport'
+        :param priority:  The priority field of the QoS
+        :type priority:   int
+        :param match:     The match rule
+        :type match:      string
+        :returns:         :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def qos_list(self, switch):
+        """Get the Qos rules for 'switch'
+
+        :param switch: The name or uuid of the switch
+        :type switch:  string or uuid.UUID
+        :returns:      :class:`Command` with RowView list result
+        """
+
     @abc.abstractmethod
     def lsp_add(self, switch, port, parent_name=None, tag=None,
                 may_exist=False, **columns):
@@ -363,6 +475,13 @@ class API(api.API):
         :returns:      :class:`Command` with RowView list result
         """
 
+    @abc.abstractmethod
+    def lr_get(self, router):
+        """Get logical router for 'router'
+
+        :returns: :class:`Command` with RowView result
+        """
+
     @abc.abstractmethod
     def lrp_add(self, router, port, mac, networks, peer=None, may_exist=False,
                 **columns):
@@ -687,12 +806,11 @@ class API(api.API):
         """
 
     @abc.abstractmethod
-    def dns_add(self, **external_ids):
-        """Create a DNS row with external_ids
+    def dns_add(self, **columns):
+        """Create a DNS row with columns
 
-        :param external_ids: external_id field key/value mapping
-        :type external_ids:  key: string, value: string
-        :returns:            :class:`Command` with RowView result
+        :param **columns:  Additional columns to directly set on the dns
+        :returns:          :class:`Command` with RowView result
         """
 
     @abc.abstractmethod
@@ -741,12 +859,15 @@ class API(api.API):
         """
 
     @abc.abstractmethod
-    def dns_remove_record(self, uuid, hostname):
+    def dns_remove_record(self, uuid, hostname, if_exists=False):
         """Remove the 'hostname' from the 'records' field of the DNS row
 
         :param uuid: The uuid of the DNS row to set the records with
         :type uuid:  string or uuid.UUID
         :param hostname: hostname as the key to the record dict
+        :param if_exists:   If True, don't fail if the DNS record
+                            doesn't exist
+        :type if_exists:    boolean
         :returns:    :class:`Command` with no result
         """
 
@@ -760,3 +881,70 @@ class API(api.API):
         :type external_ids:  key: string, value: string
         :returns:    :class:`Command` with no result
         """
+
+    @abc.abstractmethod
+    def pg_add(self, name=None, may_exist=False, **columns):
+        """Create a port group
+
+        :param name:        The name of the port group (optional)
+        :type name:         string
+        :param may_exist:   If True, don't fail if the port group already
+                            exists
+        :type may_exist:    bool
+        :param columns:     Additional columns to directly set on the port
+                            group (e.g external_ids, ports, acls)
+        :type columns:      dictionary
+        :returns:           :class:`Command` with RowView result
+        """
+
+    @abc.abstractmethod
+    def pg_del(self, name, if_exists=False):
+        """Delete a port group
+
+        :param name:        The name of the port group
+        :type name:         string
+        :param if_exists:   If True, don't fail if the port group
+                            doesn't exist
+        :type if_exists:    boolean
+        :returns:           :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def pg_add_ports(self, pg_id, lsp):
+        """Add a list of logical port to a port group
+
+        :param pg_id:     The name or uuid of the port group
+        :type pg_id:      string or uuid.UUID
+        :param lsp:       A list of :class:`Command` Logical_Switch_Port
+                          instance result or UUID
+        :type lsp:        A list of :class:`Command` Logical_Switch_Port
+                          or string or uuid.UUID
+        A Logical_Switch_Port instance or string
+                          or uuid.UUID
+        :returns:         :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def pg_del_ports(self, pg_id, lsp, if_exists=False):
+        """Delete a list of logical port from a port group
+
+        :param pg_id:     The name or uuid of the port group
+        :type pg_id:      string or uuid.UUID
+        :param lsp:       A list of :class:`Command` Logical_Switch_Port
+                          instance result or UUID
+        :type lsp:        A list of :class:`Command` Logical_Switch_Port
+                          or string or uuid.UUID
+        :type if_exists:  If True, don't fail if the logical port(s) doesn't
+                          exist
+        :type if_exists:  boolean
+        :returns:         :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def pg_get(self, pg_id):
+        """Get port group
+
+        :param pg_id: The name or uuid of the port group
+        :type pg_id:  string or uuid.UUID
+        :returns:     :class:`Command` with RowView result
+        """
