@@ -12,7 +12,6 @@
 import re
 
 import netaddr
-from oslo_utils import uuidutils
 
 from ovsdbapp.backend.ovs_idl import command as cmd
 from ovsdbapp.backend.ovs_idl import idlutils
@@ -488,7 +487,7 @@ class LspGetPortSecurityCommand(cmd.BaseCommand):
         self.result = lsp.port_security
 
 
-class LspGetUpCommand(cmd.BaseCommand):
+class LspGetUpCommand(cmd.ReadOnlyCommand):
     def __init__(self, api, port):
         super(LspGetUpCommand, self).__init__(api)
         self.port = port
@@ -510,7 +509,7 @@ class LspSetEnabledCommand(cmd.BaseCommand):
         lsp.enabled = self.is_enabled
 
 
-class LspGetEnabledCommand(cmd.BaseCommand):
+class LspGetEnabledCommand(cmd.ReadOnlyCommand):
     def __init__(self, api, port):
         super(LspGetEnabledCommand, self).__init__(api)
         self.port = port
@@ -532,7 +531,7 @@ class LspSetTypeCommand(cmd.BaseCommand):
         lsp.type = self.port_type
 
 
-class LspGetTypeCommand(cmd.BaseCommand):
+class LspGetTypeCommand(cmd.ReadOnlyCommand):
     def __init__(self, api, port):
         super(LspGetTypeCommand, self).__init__(api)
         self.port = port
@@ -553,7 +552,7 @@ class LspSetOptionsCommand(cmd.BaseCommand):
         lsp.options = self.options
 
 
-class LspGetOptionsCommand(cmd.BaseCommand):
+class LspGetOptionsCommand(cmd.ReadOnlyCommand):
     def __init__(self, api, port):
         super(LspGetOptionsCommand, self).__init__(api)
         self.port = port
@@ -574,7 +573,7 @@ class LspSetDhcpV4OptionsCommand(cmd.BaseCommand):
         lsp.dhcpv4_options = self.dhcpopt_uuid
 
 
-class LspGetDhcpV4OptionsCommand(cmd.BaseCommand):
+class LspGetDhcpV4OptionsCommand(cmd.ReadOnlyCommand):
     def __init__(self, api, port):
         super(LspGetDhcpV4OptionsCommand, self).__init__(api)
         self.port = port
@@ -611,7 +610,7 @@ class DhcpOptionsDelCommand(cmd.BaseCommand):
         dhcpopt.delete()
 
 
-class DhcpOptionsListCommand(cmd.BaseCommand):
+class DhcpOptionsListCommand(cmd.ReadOnlyCommand):
     def run_idl(self, txn):
         self.result = [rowview.RowView(r) for
                        r in self.api.tables['DHCP_Options'].rows.values()]
@@ -632,7 +631,7 @@ class DhcpOptionsSetOptionsCommand(cmd.BaseCommand):
         dhcpopt.options = self.options
 
 
-class DhcpOptionsGetOptionsCommand(cmd.BaseCommand):
+class DhcpOptionsGetOptionsCommand(cmd.ReadOnlyCommand):
     def __init__(self, api, dhcpopt_uuid):
         super(DhcpOptionsGetOptionsCommand, self).__init__(api)
         self.dhcpopt_uuid = dhcpopt_uuid
@@ -687,7 +686,7 @@ class LrDelCommand(cmd.BaseCommand):
             raise RuntimeError(msg)
 
 
-class LrListCommand(cmd.BaseCommand):
+class LrListCommand(cmd.ReadOnlyCommand):
     def run_idl(self, txn):
         self.result = [rowview.RowView(r) for
                        r in self.api.tables['Logical_Router'].rows.values()]
@@ -740,6 +739,13 @@ class LrpAddCommand(cmd.BaseCommand):
         if self.peer:
             lrp.peer = self.peer
         lr.addvalue('ports', lrp)
+        gwcs = self.columns.pop('gateway_chassis', [])
+        for n, chassis in enumerate(gwcs):
+            gwc_name = '%s_%s' % (lrp.name, chassis)
+            cmd = GatewayChassisAddCommand(self.api, gwc_name, chassis,
+                                           len(gwcs) - n, may_exist=True)
+            cmd.run_idl(txn)
+            lrp.addvalue('gateway_chassis', cmd.result)
         self.set_columns(lrp, **self.columns)
         self.result = lrp.uuid
 
@@ -757,7 +763,7 @@ class LrpDelCommand(PortDelCommand):
             if_exists)
 
 
-class LrpListCommand(cmd.BaseCommand):
+class LrpListCommand(cmd.ReadOnlyCommand):
     def __init__(self, api, router):
         super(LrpListCommand, self).__init__(api)
         self.router = router
@@ -778,7 +784,7 @@ class LrpSetEnabledCommand(cmd.BaseCommand):
         lrp.enabled = self.is_enabled
 
 
-class LrpGetEnabledCommand(cmd.BaseCommand):
+class LrpGetEnabledCommand(cmd.ReadOnlyCommand):
     def __init__(self, api, port):
         super(LrpGetEnabledCommand, self).__init__(api)
         self.port = port
@@ -859,7 +865,7 @@ class LrRouteDelCommand(cmd.BaseCommand):
             raise RuntimeError(msg)
 
 
-class LrRouteListCommand(cmd.BaseCommand):
+class LrRouteListCommand(cmd.ReadOnlyCommand):
     def __init__(self, api, router):
         super(LrRouteListCommand, self).__init__(api)
         self.router = router
@@ -968,7 +974,7 @@ class LrNatDelCommand(cmd.BaseCommand):
                                        match=self.match_ip)
 
 
-class LrNatListCommand(cmd.BaseCommand):
+class LrNatListCommand(cmd.ReadOnlyCommand):
     def __init__(self, api, router):
         super(LrNatListCommand, self).__init__(api)
         self.router = router
@@ -1035,7 +1041,7 @@ class LbDelCommand(cmd.BaseCommand):
                 raise
 
 
-class LbListCommand(cmd.BaseCommand):
+class LbListCommand(cmd.ReadOnlyCommand):
     def run_idl(self, txn):
         self.result = [rowview.RowView(r)
                        for r in self.api.tables['Load_Balancer'].rows.values()]
@@ -1080,7 +1086,7 @@ class LrLbDelCommand(cmd.BaseCommand):
             raise
 
 
-class LrLbListCommand(cmd.BaseCommand):
+class LrLbListCommand(cmd.ReadOnlyCommand):
     def __init__(self, api, router):
         super(LrLbListCommand, self).__init__(api)
         self.router = router
@@ -1129,7 +1135,7 @@ class LsLbDelCommand(cmd.BaseCommand):
             raise
 
 
-class LsLbListCommand(cmd.BaseCommand):
+class LsLbListCommand(cmd.ReadOnlyCommand):
     def __init__(self, api, switch):
         super(LsLbListCommand, self).__init__(api)
         self.switch = switch
@@ -1164,7 +1170,7 @@ class DnsGetCommand(cmd.BaseGetRowCommand):
     table = 'DNS'
 
 
-class DnsListCommand(cmd.BaseCommand):
+class DnsListCommand(cmd.ReadOnlyCommand):
     def run_idl(self, txn):
         table = self.api.tables['DNS']
         self.result = [rowview.RowView(r) for r in table.rows.values()]
@@ -1260,7 +1266,7 @@ class _PgUpdatePortsHelper(cmd.BaseCommand):
 
         if isinstance(port, cmd.BaseCommand):
             port = port.result
-        elif uuidutils.is_uuid_like(port):
+        elif utils.is_uuid_like(port):
             try:
                 port = self.api.lookup('Logical_Switch_Port', port)
             except idlutils.RowNotFound:
@@ -1292,3 +1298,30 @@ class PgDelPortCommand(_PgUpdatePortsHelper):
 
 class PgGetCommand(cmd.BaseGetRowCommand):
     table = 'Port_Group'
+
+
+class GatewayChassisAddCommand(cmd.AddCommand):
+    table_name = 'Gateway_Chassis'
+
+    def __init__(self, api, name, chassis_name, priority=0, may_exist=False,
+                 **columns):
+        super(GatewayChassisAddCommand, self).__init__(api)
+        self.name = name
+        self.chassis_name = chassis_name
+        self.priority = priority
+        self.may_exist = may_exist
+        self.columns = columns
+
+    def run_idl(self, txn):
+        if self.may_exist:
+            gwc = self.api.lookup(self.table_name, self.name, None)
+        else:
+            gwc = None
+        if not gwc:
+            # If gwc exists with name, this will properly fail if not may_exist
+            # since 'name' is indexed
+            gwc = txn.insert(self.api.tables[self.table_name])
+            gwc.name = self.name
+        gwc.priority = self.priority
+        self.set_columns(gwc, **self.columns)
+        self.result = gwc
