@@ -49,9 +49,9 @@ class OvnNbApiIdlImpl(ovs_idl.Backend, api.API):
         return self.db_add('Logical_Switch', switch_uuid, 'dns_records',
                            dns_uuid)
 
-    def ls_remove_dns_record(self, switch_uuid, dns_uuid):
+    def ls_remove_dns_record(self, switch_uuid, dns_uuid, if_exists=False):
         return self.db_remove('Logical_Switch', switch_uuid, 'dns_records',
-                              dns_uuid)
+                              dns_uuid, if_exists=if_exists)
 
     def acl_add(self, switch, direction, priority, match, action, log=False,
                 may_exist=False, **external_ids):
@@ -64,6 +64,31 @@ class OvnNbApiIdlImpl(ovs_idl.Backend, api.API):
     def acl_list(self, switch):
         return cmd.AclListCommand(self, switch)
 
+    def pg_acl_add(self, port_group, direction, priority, match, action,
+                   log=False, may_exist=False, **external_ids):
+        return cmd.PgAclAddCommand(self, port_group, direction, priority,
+                                   match, action, log, may_exist,
+                                   **external_ids)
+
+    def pg_acl_del(self, port_group, direction=None, priority=None,
+                   match=None):
+        return cmd.PgAclDelCommand(self, port_group, direction, priority,
+                                   match)
+
+    def pg_acl_list(self, port_group):
+        return cmd.PgAclListCommand(self, port_group)
+
+    def qos_add(self, switch, direction, priority, match, rate=None,
+                burst=None, dscp=None, may_exist=False, **columns):
+        return cmd.QoSAddCommand(self, switch, direction, priority, match,
+                                 rate, burst, dscp, may_exist, **columns)
+
+    def qos_del(self, switch, direction=None, priority=None, match=None):
+        return cmd.QoSDelCommand(self, switch, direction, priority, match)
+
+    def qos_list(self, switch):
+        return cmd.QoSListCommand(self, switch)
+
     def lsp_add(self, switch, port, parent_name=None, tag=None,
                 may_exist=False, **columns):
         return cmd.LspAddCommand(self, switch, port, parent_name, tag,
@@ -133,6 +158,9 @@ class OvnNbApiIdlImpl(ovs_idl.Backend, api.API):
     def lr_list(self):
         return cmd.LrListCommand(self)
 
+    def lr_get(self, router):
+        return cmd.LrGetCommand(self, router)
+
     def lrp_add(self, router, port, mac, networks, peer=None, may_exist=False,
                 **columns):
         return cmd.LrpAddCommand(self, router, port, mac, networks,
@@ -241,8 +269,24 @@ class OvnNbApiIdlImpl(ovs_idl.Backend, api.API):
             ips = " ".join(utils.normalize_ip_port(ip) for ip in ips)
         return self.db_add('DNS', uuid, 'records', {hostname: ips})
 
-    def dns_remove_record(self, uuid, hostname):
-        return self.db_remove('DNS', uuid, 'records', hostname)
+    def dns_remove_record(self, uuid, hostname, if_exists=False):
+        return self.db_remove('DNS', uuid, 'records', hostname,
+                              if_exists=if_exists)
 
     def dns_set_external_ids(self, uuid, **external_ids):
         return cmd.DnsSetExternalIdsCommand(self, uuid, **external_ids)
+
+    def pg_add(self, name=None, may_exist=False, **columns):
+        return cmd.PgAddCommand(self, name, may_exist=may_exist, **columns)
+
+    def pg_del(self, name, if_exists=False):
+        return cmd.PgDelCommand(self, name, if_exists=if_exists)
+
+    def pg_add_ports(self, pg_id, lsp):
+        return cmd.PgAddPortCommand(self, pg_id, lsp=lsp)
+
+    def pg_del_ports(self, pg_id, lsp, if_exists=False):
+        return cmd.PgDelPortCommand(self, pg_id, lsp=lsp, if_exists=if_exists)
+
+    def pg_get(self, pg):
+        return cmd.PgGetCommand(self, pg)
